"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const Realm = require("realm");
const URI = require("urijs");
class SyncedRealm {
    constructor(discovery, config) {
        this.discovery = discovery;
        this.config = config;
        const self = this;
        this.realmPromise = new Promise((resolve, reject) => {
            self.realmResolve = resolve;
        });
        if (!this.config.sync) {
            throw new Error("cannot construct SyncedRealm without `sync` configuration");
        }
        const uri = new URI(this.config.sync.url);
        const syncLabel = uri.host();
        this.discovery.find("sync", ["master", syncLabel]).then((handle) => {
            if (handle) {
                this.reconfigure(handle);
            }
        });
        this.watch = discovery.watchService("sync", ["master", syncLabel]);
        this.watch.on("available", (handle) => {
            this.reconfigure(handle);
        });
    }
    reconfigure(handle) {
        const url = new URI(this.config.sync.url).host(handle.address).port(handle.port).toString();
        if (this.url === url) {
            return;
        }
        this.url = url;
        if (this.realm) {
            console.log(`realm ${this.config.path} reconnecting to ${handle.address}:${handle.port}`);
            this.realm.syncSession._overrideServer(handle.address, handle.port);
        }
        else {
            this.realm = new Realm({
                sync: {
                    user: this.config.sync.user,
                    url,
                },
                schema: this.config.schema,
                path: this.config.path,
            });
            this.realmResolve(this.realm);
        }
    }
    open() {
        return __awaiter(this, void 0, void 0, function* () {
            return yield this.realmPromise;
        });
    }
    close() {
        if (this.realm) {
            this.realm.close();
        }
    }
}
exports.SyncedRealm = SyncedRealm;
//# sourceMappingURL=SyncedRealm.js.map